%%
% floq: compute Floquet multipliers and instability index of u_H 
%
% [muv1, muv2, ind, mo]=floq(p,jac)
% 
% muv1,muv2=stable/unstable multipliers, ind=ind(u_H), mo=monodromy M  
function [muv1, muv2, ind, mo]=floq(p,jac)
t1=tic; n1=p.hopf.nfloq; tl=p.hopf.tl; nu=p.nu; m=tl-1; 
AA=zeros(nu,nu,m); BB=AA; 
BB(:,:,1)=jac(1:nu,1:nu);
AA(:,:,1)=-jac(1:nu,(m-1)*nu+1:m*nu); 
for i=2:m; % rows of jac
  BB(:,:,i)=jac((i-1)*nu+1:i*nu, (i-1)*nu+1:i*nu); % diag 
  AA(:,:,i)=-jac((i-1)*nu+1:i*nu, (i-2)*nu+1:(i-1)*nu); % subdiag
end 
mo=speye(nu); 
for k=1:m; fak=BB(:,:,k)\AA(:,:,k); 
    mo=fak*mo; end; % building M 
ok=0; 
while ~ok; 
  [V,mu,flag]=eigs(mo,n1,'lm');  % large multipliers 
   muv=mu*ones(1,n1)'; muv=muv(isfinite(muv)); no=n1; n1=length(muv); 
  if flag==0; fprintf('All eigenvalues converged.\n'); 
  else fprintf('warning: only %i of %i requested eigenvalues converged.\n',n1,no); 
  %   fprintf('Double check with floqps!\n'); 
    % fprintf('Only returning multipliers with |ga|>0.1.\n'); 
  end 
  [muvs,ix]=sort(abs(muv)); muv=muv(ix); ok=1; 
  if abs(muv(1))>1+1e-6; 
     fprintf('min(|mu|)=%g with %i multipliers. Compute more?\n', abs(muv(1)),n1); 
     n1=min(2*n1,p.nu); n1=asknu('new n1 (0 for end)',n1); 
     if n1~=0; ok=0; end
  end
end
idx=find(abs(muv)>1+p.hopf.fltol); 
muv2=muv(idx); muv1=setdiff(muv,muv2); % the stable multipliers 
ind=length(muv2); try mu1=muv1(end); catch mu1=0; end% should be 1
mu1err=mu1-1; 
t1=toc(t1); 
if p.sw.verb>1; 
  fprintf('floq, %g seconds, error in ga_1=%g, #unst=%i\n', t1, mu1-1, ind); 
  if abs(mu1err)>p.hopf.fltol; 
  fprintf('floq: error in ga1 is probably too large ..\n'); end
end 
figure(7); clf; set(gca,'FontSize',12); 
plot(real(muv1), imag(muv1),'*b', real(muv2), imag(muv2), '*r'); 
hold on;  s=linspace(0,2*pi,100); plot(cos(s),sin(s)); axis image; %pause
end
 